@@ -10,13 +10,11 @@
 #include "Python.h"
 #include "structmember.h"
 
+#define NPY_NO_DEPRECATED_API NPY_API_VERSION
 #define _MULTIARRAYMODULE
-#include <numpy/ndarrayobject.h>
-#include <numpy/npy_3kcompat.h>
-
+#include <numpy/arrayobject.h>
 #include "npy_config.h"
-
-#include "numpy/npy_3kcompat.h"
+#include "npy_pycompat.h"
 
 typedef struct NewNpyArrayIterObject_tag NewNpyArrayIterObject;
 
@@ -169,7 +167,12 @@ NpyIter_GlobalFlagsConverter(PyObject *flags_in, npy_uint32 *flags)
                 }
                 break;
             case 'g':
-                if (strcmp(str, "growinner") == 0) {
+                /*
+                 * Documentation is grow_inner, but initial implementation
+                 * was growinner, so allowing for either.
+                 */
+                if (strcmp(str, "grow_inner") == 0 ||
+                            strcmp(str, "growinner") == 0) {
                     flag = NPY_ITER_GROWINNER;
                 }
                 break;
@@ -255,71 +258,6 @@ npyiter_order_converter(PyObject *order_in, NPY_ORDER *order)
     return 0;
 }
 
-/*NUMPY_API
- * Convert any Python object, *obj*, to an NPY_CASTING enum.
- * TODO: Move elsewhere
- */
-NPY_NO_EXPORT int
-PyArray_CastingConverter(PyObject *obj, NPY_CASTING *casting)
-{
-    char *str = NULL;
-    Py_ssize_t length = 0;
-
-    if (PyUnicode_Check(obj)) {
-        PyObject *str_obj;
-        int ret;
-        str_obj = PyUnicode_AsASCIIString(obj);
-        if (str_obj == NULL) {
-            return 0;
-        }
-        ret = PyArray_CastingConverter(str_obj, casting);
-        Py_DECREF(str_obj);
-        return ret;
-    }
-
-    if (PyBytes_AsStringAndSize(obj, &str, &length) == -1) {
-        return 0;
-    }
-
-    if (length >= 2) switch (str[2]) {
-        case 0:
-            if (strcmp(str, "no") == 0) {
-                *casting = NPY_NO_CASTING;
-                return 1;
-            }
-            break;
-        case 'u':
-            if (strcmp(str, "equiv") == 0) {
-                *casting = NPY_EQUIV_CASTING;
-                return 1;
-            }
-            break;
-        case 'f':
-            if (strcmp(str, "safe") == 0) {
-                *casting = NPY_SAFE_CASTING;
-                return 1;
-            }
-            break;
-        case 'm':
-            if (strcmp(str, "same_kind") == 0) {
-                *casting = NPY_SAME_KIND_CASTING;
-                return 1;
-            }
-            break;
-        case 's':
-            if (strcmp(str, "unsafe") == 0) {
-                *casting = NPY_UNSAFE_CASTING;
-                return 1;
-            }
-            break;
-    }
-
-    PyErr_SetString(PyExc_ValueError,
-            "casting must be one of 'no', 'equiv', 'safe', "
-            "'same_kind', or 'unsafe'");
-    return 0;
-}
-
 static int
 NpyIter_OpFlagsConverter(PyObject *op_flags_in,
                          npy_uint32 *op_flags)
@@ -369,11 +307,22 @@ NpyIter_OpFlagsConverter(PyObject *op_flags_in,
         flag = 0;
         switch (str[0]) {
             case 'a':
-                if (strcmp(str, "allocate") == 0) {
-                    flag = NPY_ITER_ALLOCATE;
-                }
-                if (strcmp(str, "aligned") == 0) {
-                    flag = NPY_ITER_ALIGNED;
+                if (length > 2) switch(str[2]) {
+                    case 'i':
+                        if (strcmp(str, "aligned") == 0) {
+                            flag = NPY_ITER_ALIGNED;
+                        }
+                        break;
+                    case 'l':
+                        if (strcmp(str, "allocate") == 0) {
+                            flag = NPY_ITER_ALLOCATE;
+                        }
+                        break;
+                    case 'r':
+                        if (strcmp(str, "arraymask") == 0) {
+                            flag = NPY_ITER_ARRAYMASK;
+                        }
+                        break;
                 }
                 break;
             case 'c':
@@ -416,13 +365,31 @@ NpyIter_OpFlagsConverter(PyObject *op_flags_in,
                 }
                 break;
             case 'u':
-                if (strcmp(str, "updateifcopy") == 0) {
-                    flag = NPY_ITER_UPDATEIFCOPY;
+                switch (str[1]) {
+                    case 'p':
+                        if (strcmp(str, "updateifcopy") == 0) {
+                            flag = NPY_ITER_UPDATEIFCOPY;
+                        }
+                        break;
+                }
+                break;
+            case 'v':
+                if (strcmp(str, "virtual") == 0) {
+                    flag = NPY_ITER_VIRTUAL;
                 }
                 break;
             case 'w':
-                if (strcmp(str, "writeonly") == 0) {
-                    flag = NPY_ITER_WRITEONLY;
+                if (length > 5) switch (str[5]) {
+                    case 'o':
+                        if (strcmp(str, "writeonly") == 0) {
+                            flag = NPY_ITER_WRITEONLY;
+                        }
+                        break;
+                    case 'm':
+                        if (strcmp(str, "writemasked") == 0) {
+                            flag = NPY_ITER_WRITEMASKED;
+                        }
+                        break;
                 }
                 break;
         }
@@ -633,9 +600,9 @@ npyiter_convert_op_axes(PyObject *op_axes_in, npy_intp nop,
 }
 
 /*
- * Converts the operand array and op_flags array into the form NpyIter_AdvancedNew
- * needs.  Sets nop, and on success, each op[i] owns a reference
- * to an array object.
+ * Converts the operand array and op_flags array into the form
+ * NpyIter_AdvancedNew needs.  Sets nop, and on success, each
+ * op[i] owns a reference to an array object.
  */
 static int
 npyiter_convert_ops(PyObject *op_in, PyObject *op_flags_in,
@@ -715,7 +682,7 @@ npyiter_convert_ops(PyObject *op_in, PyObject *op_flags_in,
             int fromanyflags = 0;
 
             if (op_flags[iop]&(NPY_ITER_READWRITE|NPY_ITER_WRITEONLY)) {
-                fromanyflags = NPY_UPDATEIFCOPY;
+                fromanyflags |= NPY_ARRAY_UPDATEIFCOPY;
             }
             ao = (PyArrayObject *)PyArray_FromAny((PyObject *)op[iop],
                                             NULL, 0, 0, fromanyflags, NULL);
@@ -833,6 +800,7 @@ npyiter_init(NewNpyArrayIterObject *self, PyObject *args, PyObject *kwds)
         itershape.ptr = NULL;
     }
 
+
     self->iter = NpyIter_AdvancedNew(nop, op, flags, order, casting, op_flags,
                                   op_request_dtypes,
                                   oa_ndim, oa_ndim > 0 ? op_axes : NULL,
@@ -1603,7 +1571,8 @@ static PyObject *npyiter_multi_index_get(NewNpyArrayIterObject *self)
     }
 }
 
-static int npyiter_multi_index_set(NewNpyArrayIterObject *self, PyObject *value)
+static int
+npyiter_multi_index_set(NewNpyArrayIterObject *self, PyObject *value)
 {
     npy_intp idim, ndim, multi_index[NPY_MAXDIMS];
 
@@ -1984,12 +1953,14 @@ npyiter_seq_length(NewNpyArrayIterObject *self)
 NPY_NO_EXPORT PyObject *
 npyiter_seq_item(NewNpyArrayIterObject *self, Py_ssize_t i)
 {
-    PyObject *ret;
+    PyArrayObject *ret;
 
     npy_intp ret_ndim;
     npy_intp nop, innerloopsize, innerstride;
     char *dataptr;
     PyArray_Descr *dtype;
+    int has_external_loop;
+    Py_ssize_t i_orig = i;
 
     if (self->iter == NULL || self->finished) {
         PyErr_SetString(PyExc_ValueError,
@@ -2005,9 +1976,15 @@ npyiter_seq_item(NewNpyArrayIterObject *self, Py_ssize_t i)
     }
 
     nop = NpyIter_GetNOp(self->iter);
+
+    /* Negative indexing */
+    if (i < 0) {
+        i += nop;
+    }
+
     if (i < 0 || i >= nop) {
         PyErr_Format(PyExc_IndexError,
-                "Iterator operand index %d is out of bounds", (int)i);
+                "Iterator operand index %d is out of bounds", (int)i_orig);
         return NULL;
     }
 
@@ -2028,8 +2005,9 @@ npyiter_seq_item(NewNpyArrayIterObject *self, Py_ssize_t i)
 
     dataptr = self->dataptrs[i];
     dtype = self->dtypes[i];
+    has_external_loop = NpyIter_HasExternalLoop(self->iter);
 
-    if (NpyIter_HasExternalLoop(self->iter)) {
+    if (has_external_loop) {
         innerloopsize = *self->innerloopsizeptr;
         innerstride = self->innerstrides[i];
         ret_ndim = 1;
@@ -2042,16 +2020,19 @@ npyiter_seq_item(NewNpyArrayIterObject *self, Py_ssize_t i)
     }
 
     Py_INCREF(dtype);
-    ret = (PyObject *)PyArray_NewFromDescr(&PyArray_Type, dtype,
-                            ret_ndim, &innerloopsize,
-                            &innerstride, dataptr,
-                            self->writeflags[i] ? NPY_WRITEABLE : 0, NULL);
+    ret = (PyArrayObject *)PyArray_NewFromDescr(&PyArray_Type, dtype,
+                        ret_ndim, &innerloopsize,
+                        &innerstride, dataptr,
+                        self->writeflags[i] ? NPY_ARRAY_WRITEABLE : 0, NULL);
     Py_INCREF(self);
-    ((PyArrayObject *)ret)->base = (PyObject *)self;
+    if (PyArray_SetBaseObject(ret, (PyObject *)self) < 0) {
+        Py_DECREF(ret);
+        return NULL;
+    }
 
-    PyArray_UpdateFlags((PyArrayObject *)ret, NPY_UPDATE_ALL);
+    PyArray_UpdateFlags(ret, NPY_ARRAY_UPDATE_ALL);
 
-    return ret;
+    return (PyObject *)ret;
 }
 
 NPY_NO_EXPORT PyObject *
@@ -2112,7 +2093,9 @@ npyiter_seq_ass_item(NewNpyArrayIterObject *self, Py_ssize_t i, PyObject *v)
     char *dataptr;
     PyArray_Descr *dtype;
     PyArrayObject *tmp;
-    int ret;
+    int ret, has_external_loop;
+    Py_ssize_t i_orig = i;
+
 
     if (v == NULL) {
         PyErr_SetString(PyExc_TypeError,
@@ -2134,21 +2117,28 @@ npyiter_seq_ass_item(NewNpyArrayIterObject *self, Py_ssize_t i, PyObject *v)
     }
 
     nop = NpyIter_GetNOp(self->iter);
+
+    /* Negative indexing */
+    if (i < 0) {
+        i += nop;
+    }
+
     if (i < 0 || i >= nop) {
         PyErr_Format(PyExc_IndexError,
-                "Iterator operand index %d is out of bounds", (int)i);
+                "Iterator operand index %d is out of bounds", (int)i_orig);
         return -1;
     }
     if (!self->writeflags[i]) {
         PyErr_Format(PyExc_RuntimeError,
-                "Iterator operand %d is not writeable", (int)i);
+                "Iterator operand %d is not writeable", (int)i_orig);
         return -1;
     }
 
     dataptr = self->dataptrs[i];
     dtype = self->dtypes[i];
+    has_external_loop = NpyIter_HasExternalLoop(self->iter);
 
-    if (NpyIter_HasExternalLoop(self->iter)) {
+    if (has_external_loop) {
         innerloopsize = *self->innerloopsizeptr;
         innerstride = self->innerstrides[i];
     }
@@ -2162,11 +2152,13 @@ npyiter_seq_ass_item(NewNpyArrayIterObject *self, Py_ssize_t i, PyObject *v)
     tmp = (PyArrayObject *)PyArray_NewFromDescr(&PyArray_Type, dtype,
                                 1, &innerloopsize,
                                 &innerstride, dataptr,
-                                NPY_WRITEABLE, NULL);
+                                NPY_ARRAY_WRITEABLE, NULL);
     if (tmp == NULL) {
         return -1;
     }
-    PyArray_UpdateFlags(tmp, NPY_UPDATE_ALL);
+
+    PyArray_UpdateFlags(tmp, NPY_ARRAY_UPDATE_ALL);
+
     ret = PyArray_CopyObject(tmp, v);
     Py_DECREF(tmp);
     return ret;
@@ -2402,7 +2394,7 @@ static PyGetSetDef npyiter_getsets[] = {
         (getter)npyiter_finished_get,
         NULL, NULL, NULL},
 
-    {NULL, NULL, NULL, NULL, NULL},
+    {NULL, NULL, NULL, NULL, NULL}
 };
 
 NPY_NO_EXPORT PySequenceMethods npyiter_as_sequence = {
